package TextData::Calc;

#TextData version 1.6.0 (7.3.2006)
#see POD below!

use Math::Trig;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(LinearValue ParsedFormula FormulaFunction);

#LinearValue(x,[x1,x2],[y1,y2])
sub LinearValue
{
	my ($x,$ox,$oy) = @_;
	return $ox->[0] != $ox->[1] ? ( $oy->[0] + ($oy->[1]-$oy->[0])*($x-$ox->[0])/($ox->[1]-$ox->[0]) ) : ( $x == $ox->[0] ? ($oy->[0]+$oy->[1])/2 : undef );
}

#ParsedFormula(text, dataarrayname, pararrayname1, pararrayname2)
#A -> pararray1
#C -> pararray2
#[a,b] -> data[a][b]

sub ParsedFormula
{
	my ($formula, $data, $par1, $par2) = @_;
	
	my $nf = '';
	
	#use loop because probably needing to use $2 as $1 in next run...
	#some more pure regex magic should do it in future...
	#$ord$ord is not translated correctly but is not correct syntax anyway
	while($formula =~ s/([^a-zA-Z\$]|^)\$?a([^(a-zA-Z]|$)/$1\[0,2\]$2/m){};
	while($formula =~ s/([^a-zA-Z\$]|^)\$?b([^(a-zA-Z]|$)/$1\[1,2\]$2/m){};
	while($formula =~ s/([^a-zA-Z\$]|^)\$?x([^(a-zA-Z]|$)/$1\[0,1\]$2/m){};
	$formula =~ s/\$ord([^(a-zA-Z]|$)/\[0,1\]$1/g;
	while($formula =~ s/([^a-zA-Z\$]|^)\$?y([^(a-zA-Z]|$)/$1\[0,2\]$2/m){};
	while($formula =~ s/\A([^[]*[^[a-zA-Z]|)\[([\d,]+)\]//)
	{
		$nf .= $1;
		my $num = $2;
		if( $num =~ /^(\d+),(\d+)$/ )
		{
			return undef if $1 < 0 or $2 < 1;
			$nf .= '$'.$data."[$1][".($2-1).']';		
		}
		else
		{
			return undef if $num < 1;
			$nf .= '$'.$data."[0][".($num-1).']';		
		}
	}

	$nf .= $formula;
	$nf =~ s/(^|[^\$a-zA-Z])A(\d+)/$1\$$par1\[$2\]/g;	
	$nf =~ s/(^|[^\$a-zA-Z])C(\d+)/$1\$$par2\[$2\]/g;	
	return $nf;
}

#(formula, be_verbose)
sub FormulaFunction
{
	my ($formula,$verb) = @_;
	my $pf = ParsedFormula($formula, 'fd->', 'A->', 'C->');
	print STDERR "Formula code: ",$pf,"\n" if $verb;
	return eval 'sub { my ($fd, $A, $C) = @_; '.$pf.' }';
}

1;
__END__

=head1 NAME

TextData::Calc -  helper package for some calculations

=head1 SYNOPSIS

	use TextData::Calc qw(LinearValue FormulaFunction);
		
	#inter- or extrapolation with known data points ($x1,$y1) and ($x2,$y2)
	$y = LinearValue($x,[$x1,$x2],[$y1,$y2]);
	my $ff = FormulaFunction($textualformula);
	my $value = &$ff(\@datasets);

=head1 DESCRIPTION

This is a little library for TextData; it contains routines that in fact are too general to strictly belong to TextData... but here they are.

=head2 Functions

=over * 4

=item * LinearValue($x,[$x1,$x2],[$y1,$y2]) -> $y

does simple linear inter-/extrapolation of the $y for a given $x based on two known points. It is needed by the TextData::File class for, well, interpolation.

=item * FormulaFunction($formula) -> \&function

This takes some formula as text and constructs a function that evaluates this formula. The created function takes three arrays (references) as arguments: A two-dimensional array and two one-dimensional arrays with some data needed for calculation.
This facility is created for and used by the program named txdcalc to provide flexible data manipulation. So, see that one for formula syntax and semantics.

=item * ParsedFormula($formula, $dataname, $par1name, $par2name) -> $perl_code

is the parser for the $formula that just replaces the different short forms for data and constants with proper PERL code using the last three parameters for the array names. It's used internally by FormulaFunction. If you really want to use it explicitly, then it shouldn't be hard to figure the usage out by looking at the source of FormulaFunction.

=back

=head1 LICENSE

This package is open source software under the terms of the Artistic License. Free use respecting the author;-)

=head1 AUTHOR

Thomas Orgis <thomas@orgis.org>

=head1 WEB

See	http://thomas.orgis.org/scripterei for updates and lots of other (un)related stuff.
